package common.functions.primitive;

import java.util.Objects;

/**
 * A functional interface for a consumer that accepts a triple 
 * (int s, int t, double d).
 * <br>
 * In PRISM, this interface is mostly used in the context that
 * s is the index of an outgoing state, t is the index of the destination
 * state and d is the probability / rate of the transition from s to t. 
 */
@FunctionalInterface
public interface IntIntDoubleConsumer
{
	/**
	 * Performs this operation on the given arguments.
	 */
	public void accept(int s, int t, double d);

	/**
	 * Returns a composed {@code IntIntDoubleConsumer} that performs, in sequence, this
	 * operation followed by the {@code after} operation (see to {@link java.util.function.Consumer#andThen}).
	 */
	default IntIntDoubleConsumer andThen(IntIntDoubleConsumer after) {
		Objects.requireNonNull(after);
		return (int s, int t, double d) -> { accept(s,t,d); after.accept(s,t,d); };
	}
}
